﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Reflection;
using System.Text.RegularExpressions;

namespace HeuristicLab.Problems.Instances.CFSAP {
  public class BozejkoCFSAPInstanceProvider : ProblemInstanceProvider<CFSAPData> {

    public override string Name {
      get { return "Bozejko (random CFSAP)"; }
    }

    public override string Description {
      get { return string.Empty; }
    }

    public override Uri WebLink {
      get { return null; }
    }

    public override string ReferencePublication {
      get { return string.Empty; }
    }

    public override bool CanImportData {
      get { return true; }
    }

    private CFSAPData LoadInstance(Stream stream) {
      var parser = new BozejkoCFSAPParser();
      parser.Parse(stream);
      return new CFSAPData {
        Jobs = parser.Jobs,
        MachineNests = 2,
        ProcessingTimes = parser.ProcessingTimes,
        SetupTimes = parser.SetupTimes
      };
    }
    public override CFSAPData ImportData(string path) {
      var instance = LoadInstance(new FileStream(path, FileMode.Open, FileAccess.Read, FileShare.ReadWrite));
      instance.Name = Path.GetFileName(path);
      instance.Description = "Imported from " + path + " " + DateTime.Now;
      return instance;
    }

    public override IEnumerable<IDataDescriptor> GetDataDescriptors() {
      var instanceArchiveName = GetResourceName(@"BozejkoCFSAP\.zip");
      if (String.IsNullOrEmpty(instanceArchiveName)) yield break;

      using (var instanceStream = new ZipArchive(GetType().Assembly.GetManifestResourceStream(instanceArchiveName), ZipArchiveMode.Read)) {
        foreach (var entry in instanceStream.Entries.Select(x => x.Name).OrderBy(x => x)) {
          yield return new BozejkoCFSAPDataDescriptor(Path.GetFileNameWithoutExtension(entry),
                                                      GetInstanceDescription(), entry);
        }
      }
    }

    public override CFSAPData LoadData(IDataDescriptor id) {
      var descriptor = (BozejkoCFSAPDataDescriptor)id;
      var instanceArchiveName = GetResourceName(@"BozejkoCFSAP\.zip");
      using (var instancesZipFile = new ZipArchive(GetType().Assembly.GetManifestResourceStream(instanceArchiveName), ZipArchiveMode.Read)) {
        var entry = instancesZipFile.GetEntry(descriptor.InstanceIdentifier);
        var instance = LoadInstance(entry.Open());
        instance.Name = id.Name;
        instance.Description = id.Description;
        return instance;
      }
    }

    private string GetResourceName(string fileName) {
      return Assembly.GetExecutingAssembly()
                     .GetManifestResourceNames()
                     .SingleOrDefault(x => Regex.Match(x, @".*\.Data\." + fileName).Success);
    }

    private string GetInstanceDescription() {
      return "Embedded instance of plugin version " + Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyFileVersionAttribute), true).Cast<AssemblyFileVersionAttribute>().First().Version + ".";
    }

  }
}
